/* -*- mode: c++; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
 * Copyright (C) 2015  HUBzero Foundation, LLC
 *
 * Author: Leif Delgass <ldelgass@purdue.edu>
 */
#include "ColorMap.h"

using namespace GeoVis;

void
ColorMap::renderColorMap(osg::TransferFunction1D *map, int width, int height,
                         osg::Image *imgData,
                         bool opaque, float bgColor[3],
                         bool bgr,
                         int bytesPerPixel)
{
    int size = bytesPerPixel * width * height;
    if (imgData->getTotalSizeInBytes() != (unsigned int)size) {
        GLenum pixelFormat;
        if (bytesPerPixel == 3) {
            pixelFormat = bgr ? GL_BGR : GL_RGB;
        } else {
            pixelFormat = bgr ? GL_BGRA: GL_RGBA;
        }; 
        imgData->allocateImage(width, height, 1, pixelFormat, GL_UNSIGNED_BYTE, 1);
    }
    unsigned char *dst = imgData->data();
    //unsigned char *src = map->data();
    float min = map->getMinimum();
    float max = map->getMaximum();
    if (height > width) {
        for (int i = 0; i < height; i++) {
            float x = (float)i/(height-1);
            float val = min + (max - min) * x;
            osg::Vec4f rgba = map->getColor(val);
            float opacity = rgba[3];
            unsigned char color[3];
            if (opaque) {
                color[0] = (unsigned char)(255. * (bgr ? rgba[2] : rgba[0]));
                color[1] = (unsigned char)(255. * rgba[1]);
                color[2] = (unsigned char)(255. * (bgr ? rgba[0] : rgba[2]));
            } else {
                color[0] = (unsigned char)(255. * (bgColor[0] * (1.0 - opacity) + (bgr ? rgba[2] : rgba[0]) * opacity));
                color[1] = (unsigned char)(255. * (bgColor[1] * (1.0 - opacity) + rgba[1] * opacity));
                color[2] = (unsigned char)(255. * (bgColor[2] * (1.0 - opacity) + (bgr ? rgba[0] : rgba[2]) * opacity));
            }
            for (int j = 0; j < width; j++) {
                memcpy(dst, color, 3);
                dst += 3;
                if (bytesPerPixel == 4) {
                    *(dst++) = (unsigned char)(255. * (opaque ? 1.0 : opacity));
                }
            }
        }
    } else {
        for (int i = 0; i < height; i++) {
            for (int j = 0; j < width; j++) {
                float x = (float)j/(width-1);
                float val = min + (max - min) * x;
                osg::Vec4f rgb = map->getColor(val);
                double opacity = rgb[3];
                unsigned char color[3];
                if (opaque) {
                    color[0] = (unsigned char)(255. * (bgr ? rgb[2] : rgb[0]));
                    color[1] = (unsigned char)(255. * rgb[1]);
                    color[2] = (unsigned char)(255. * (bgr ? rgb[0] : rgb[2]));
                } else {
                    color[0] = (unsigned char)(255. * (bgColor[0] * (1.0 - opacity) + (bgr ? rgb[2] : rgb[0]) * opacity));
                    color[1] = (unsigned char)(255. * (bgColor[1] * (1.0 - opacity) + rgb[1] * opacity));
                    color[2] = (unsigned char)(255. * (bgColor[2] * (1.0 - opacity) + (bgr ? rgb[0] : rgb[2]) * opacity));
                }
                memcpy(dst, color, 3);
                dst += 3;
                if (bytesPerPixel == 4) {
                    *(dst++) = (unsigned char)(255. * (opaque ? 1.0 : opacity));
                }
            }
        }
    }
}
