/* -*- mode: c++; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
 * Copyright (C) 2015  HUBzero Foundation, LLC
 *
 * Author: Leif Delgass <ldelgass@purdue.edu>
 */
#ifndef GEOVIS_IDATA_H
#define GEOVIS_IDATA_H

#include <vector>
#include <string>

namespace IData {

enum Command {
    COMMAND_LIST,
    COMMAND_GET,
    COMMAND_CONTENTS,
    COMMAND_SETGLOBALMETA,
    COMMAND_DELETE,
    COMMAND_UPLOAD,
    COMMAND_FINISHUPLOAD,
    COMMAND_CHECKSUM
};

/// For "shared" property of Collection
enum Shared {
    NOT_SHARED,
    READ_ONLY,
    READ_WRITE
};

/// For "dir-or-file" property of DirectoryItem
enum DirectoryItemType {
    TYPE_DIRECTORY = 1,
    TYPE_FILE = 2
};

class Collection {
public:
    std::string name;
    std::string shared; // "no", "read-only","read-write"
    unsigned int id;
};

class DirectoryItem {
public:
    DirectoryItem() :
        size(0)
    {}
    std::string name;
    bool isDir;
    std::string type;
    size_t size;
    std::string ctime;
    std::string doi;
};

class Buffer {
public:
    Buffer() :
      data(0), size(0)
    {}
    ~Buffer() { free(data); }
    void *data;
    size_t size;
};

extern void iDataInit(const char *username, const char *hubURL = NULL);
extern void iDataCleanup();
extern void getCollectionList(std::vector<Collection>& list);
extern void getContents(int collectionID, const char *path, std::vector<DirectoryItem>& contents);
extern void getFile(int collectionID, const char *doi, Buffer *buff);
extern void deleteFile(int collectionID, const char *path, const char *filename);
extern void setFileMetadata(int collectionID, const char *doi, const char *name, const char *value);

}

#endif
