/* -*- mode: c++; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
 * Copyright (C) 2015  HUBzero Foundation, LLC
 *
 * Author: Leif Delgass <ldelgass@purdue.edu>
 */

#include <string>
#include <fstream>

#include <osgDB/FileUtils>
#include <osgDB/FileNameUtils>

#include "Util.h"
#include "Trace.h"

using namespace GeoVis;

std::string GeoVis::loadCSVLayer(const char *csvFilePath, const char *longitudeColName, const char *latitudeColName)
{
    std::string baseName = osgDB::getStrippedName(csvFilePath);
    std::string vrtFile = osgDB::getNameLessExtension(csvFilePath) + std::string(".vrt");
    if (!osgDB::fileExists(vrtFile)) {
        std::ofstream ofs(vrtFile.c_str());
        if (ofs.is_open()) {
            // GDAL/OGR < 1.9.2 doesn't like the <?xml?> declaration
            // << "<?xml version=\"1.0\"?>" << std::endl
            ofs << "<OGRVRTDataSource>" << std::endl
                << " <OGRVRTLayer name=\"" << baseName << "\">" << std::endl
                << "  <SrcDataSource>" << csvFilePath << "</SrcDataSource>" << std::endl
                << "  <GeometryType>wkbPoint</GeometryType>" << std::endl
                << "  <LayerSRS>WGS84</LayerSRS>" << std::endl
                << "  <GeometryField encoding=\"PointFromColumns\" x=\""
                << longitudeColName << "\" y=\""  << latitudeColName << "\"/>" << std::endl
                << " </OGRVRTLayer>" << std::endl
                << "</OGRVRTDataSource>" << std::endl;
            ofs.close();
        } else {
            ERROR("Failed to open file '%s' for writing", vrtFile.c_str());
        }
    } else {
        TRACE("Found existing file  '%s'", vrtFile.c_str());
    }
    return vrtFile;
}
